@@ -21,9 +21,13 @@ import string
 import tempfile
 
 from swift import gettext_ as _
-from exceptions import PLOTLIBNotInstalled, ODFLIBNotInstalled,\
-    NotFoundException, MethodNotAllowed, DataLoadFailure, ProfileException
-from profile_model import Stats2
+from swift.common.middleware.x_profile.exceptions import PLOTLIBNotInstalled
+from swift.common.middleware.x_profile.exceptions import ODFLIBNotInstalled
+from swift.common.middleware.x_profile.exceptions import NotFoundException
+from swift.common.middleware.x_profile.exceptions import MethodNotAllowed
+from swift.common.middleware.x_profile.exceptions import DataLoadFailure
+from swift.common.middleware.x_profile.exceptions import ProfileException
+from swift.common.middleware.x_profile.profile_model import Stats2
 
 PLOTLIB_INSTALLED = True
 try:
@@ -319,8 +323,7 @@ class HTMLViewer(object):
         if not fulldirs:
             stats.strip_dirs()
         stats.sort_stats(sort)
-        nfl_filter_esc =\
-            nfl_filter.replace('(', '\(').replace(')', '\)')
+        nfl_filter_esc = nfl_filter.replace('(', '\(').replace(')', '\)')
         amount = [nfl_filter_esc, limit] if nfl_filter_esc else [limit]
         profile_html = self.generate_stats_html(stats, self.app_path,
                                                 profile_id, *amount)
