@@ -401,6 +401,7 @@ class ObjectReconstructor(Daemon):
                        path, headers, full_get_path)
 
         buckets = defaultdict(dict)
+        durable_buckets = {}
         etag_buckets = {}
         error_resp_count = 0
         for resp in pile:
@@ -444,6 +445,10 @@ class ObjectReconstructor(Daemon):
                 continue
             timestamp = Timestamp(timestamp)
 
+            durable = resp.headers.get('X-Backend-Durable-Timestamp')
+            if durable:
+                durable_buckets[Timestamp(durable)] = True
+
             etag = resp.headers.get('X-Object-Sysmeta-Ec-Etag')
             if not etag:
                 self.logger.warning('Invalid resp from %s, frag index %s '
@@ -463,32 +468,35 @@ class ObjectReconstructor(Daemon):
             if resp_frag_index not in buckets[timestamp]:
                 buckets[timestamp][resp_frag_index] = resp
                 if len(buckets[timestamp]) >= job['policy'].ec_ndata:
-                    responses = buckets[timestamp].values()
+                    responses = list(buckets[timestamp].values())
                     self.logger.debug(
                         'Reconstruct frag #%s with frag indexes %s'
                         % (fi_to_rebuild, list(buckets[timestamp])))
                     break
         else:
+            path = _full_path(node, job['partition'],
+                              datafile_metadata['name'],
+                              job['policy'])
+
             for timestamp, resp in sorted(buckets.items()):
                 etag = etag_buckets[timestamp]
+                durable = durable_buckets.get(timestamp)
                 self.logger.error(
-                    'Unable to get enough responses (%s/%s) '
-                    'to reconstruct %s frag#%s with ETag %s' % (
+                    'Unable to get enough responses (%s/%s) to reconstruct '
+                    '%s %s frag#%s with ETag %s and timestamp %s' % (
                         len(resp), job['policy'].ec_ndata,
-                        _full_path(node, job['partition'],
-                                   datafile_metadata['name'],
-                                   job['policy']),
-                        fi_to_rebuild, etag))
+                        'durable' if durable else 'non-durable',
+                        path, fi_to_rebuild, etag, timestamp.internal))
 
             if error_resp_count:
+                durable = durable_buckets.get(Timestamp(
+                    datafile_metadata['X-Timestamp']))
                 self.logger.error(
                     'Unable to get enough responses (%s error responses) '
-                    'to reconstruct %s frag#%s' % (
+                    'to reconstruct %s %s frag#%s' % (
                         error_resp_count,
-                        _full_path(node, job['partition'],
-                                   datafile_metadata['name'],
-                                   job['policy']),
-                        fi_to_rebuild))
+                        'durable' if durable else 'non-durable',
+                        path, fi_to_rebuild))
 
             raise DiskFileError('Unable to reconstruct EC archive')
 
@@ -509,15 +517,15 @@ class ObjectReconstructor(Daemon):
         """
 
         def _get_one_fragment(resp):
-            buff = ''
+            buff = []
             remaining_bytes = policy.fragment_size
             while remaining_bytes:
                 chunk = resp.read(remaining_bytes)
                 if not chunk:
                     break
                 remaining_bytes -= len(chunk)
-                buff += chunk
-            return buff
+                buff.append(chunk)
+            return b''.join(buff)
 
         def fragment_payload_iter():
             # We need a fragment from each connections, so best to
@@ -706,7 +714,7 @@ class ObjectReconstructor(Daemon):
         local suffix's hashes to decide which suffixes (if any) are out
         of sync.
 
-        :param: the job dict, with the keys defined in ``_get_part_jobs``
+        :param job: the job dict, with the keys defined in ``_get_part_jobs``
         :param node: the remote node dict
         :returns: a (possibly empty) list of strings, the suffixes to be
                   synced and the remote node.
@@ -821,7 +829,7 @@ class ObjectReconstructor(Daemon):
         because each job will define a separate list of node(s) to
         'sync_to'.
 
-        :param: the job dict, with the keys defined in ``_get_job_info``
+        :param job: the job dict, with the keys defined in ``_get_job_info``
         """
         begin = time.time()
         if job['job_type'] == REVERT:
@@ -850,7 +858,8 @@ class ObjectReconstructor(Daemon):
             success, _ = ssync_sender(
                 self, node, job, suffixes)()
             # let remote end know to rehash it's suffixes
-            self.rehash_remote(node, job, suffixes)
+            if success:
+                self.rehash_remote(node, job, suffixes)
             # update stats for this attempt
             self.suffix_sync += len(suffixes)
             self.logger.update_stats('suffix.syncs', len(suffixes))
