@@ -16,9 +16,10 @@
 from swift import gettext_ as _
 import json
 
+import six
 from six.moves.urllib.parse import unquote
 
-from swift.common.utils import public, csv_append, Timestamp, \
+from swift.common.utils import public, private, csv_append, Timestamp, \
     config_true_value, ShardRange
 from swift.common.constraints import check_metadata, CONTAINER_LISTING_LIMIT
 from swift.common.http import HTTP_ACCEPTED, is_success
@@ -27,7 +28,8 @@ from swift.proxy.controllers.base import Controller, delay_denial, \
     cors_validation, set_info_cache, clear_info_cache
 from swift.common.storage_policy import POLICIES
 from swift.common.swob import HTTPBadRequest, HTTPForbidden, \
-    HTTPNotFound, HTTPServiceUnavailable, str_to_wsgi, wsgi_to_bytes
+    HTTPNotFound, HTTPServiceUnavailable, str_to_wsgi, wsgi_to_str, \
+    bytes_to_wsgi
 
 
 class ContainerController(Controller):
@@ -162,8 +164,8 @@ class ContainerController(Controller):
         params.pop('states', None)
         req.headers.pop('X-Backend-Record-Type', None)
         reverse = config_true_value(params.get('reverse'))
-        marker = params.get('marker')
-        end_marker = params.get('end_marker')
+        marker = wsgi_to_str(params.get('marker'))
+        end_marker = wsgi_to_str(params.get('end_marker'))
 
         limit = req_limit
         for shard_range in shard_ranges:
@@ -176,9 +178,9 @@ class ContainerController(Controller):
             if objects:
                 last_name = objects[-1].get('name',
                                             objects[-1].get('subdir', u''))
-                params['marker'] = last_name.encode('utf-8')
+                params['marker'] = bytes_to_wsgi(last_name.encode('utf-8'))
             elif marker:
-                params['marker'] = marker
+                params['marker'] = str_to_wsgi(marker)
             else:
                 params['marker'] = ''
             # Always set end_marker to ensure that misplaced objects beyond the
@@ -186,7 +188,7 @@ class ContainerController(Controller):
             # object obscuring correctly placed objects in the next shard
             # range.
             if end_marker and end_marker in shard_range:
-                params['end_marker'] = end_marker
+                params['end_marker'] = str_to_wsgi(end_marker)
             elif reverse:
                 params['end_marker'] = str_to_wsgi(shard_range.lower_str)
             else:
@@ -213,13 +215,13 @@ class ContainerController(Controller):
 
             if limit <= 0:
                 break
-            if (end_marker and reverse and
-                (wsgi_to_bytes(end_marker) >=
-                 objects[-1]['name'].encode('utf-8'))):
+            last_name = objects[-1].get('name',
+                                        objects[-1].get('subdir', u''))
+            if six.PY2:
+                last_name = last_name.encode('utf8')
+            if end_marker and reverse and end_marker >= last_name:
                 break
-            if (end_marker and not reverse and
-                (wsgi_to_bytes(end_marker) <=
-                 objects[-1]['name'].encode('utf-8'))):
+            if end_marker and not reverse and end_marker <= last_name:
                 break
 
         resp.body = json.dumps(objects).encode('ascii')
@@ -270,9 +272,9 @@ class ContainerController(Controller):
                 str(config_true_value(req.headers['X-Container-Sharding']))
         length_limit = self.get_name_length_limit()
         if len(self.container_name) > length_limit:
-            resp = HTTPBadRequest(request=req)
-            resp.body = 'Container name length of %d longer than %d' % \
-                        (len(self.container_name), length_limit)
+            body = 'Container name length of %d longer than %d' % (
+                len(self.container_name), length_limit)
+            resp = HTTPBadRequest(request=req, body=body)
             return resp
         account_partition, accounts, container_count = \
             self.account_info(self.account_name, req)
@@ -289,9 +291,9 @@ class ContainerController(Controller):
                 self.container_info(self.account_name, self.container_name,
                                     req)
             if not is_success(container_info.get('status')):
-                resp = HTTPForbidden(request=req)
-                resp.body = 'Reached container limit of %s' % \
-                    self.app.max_containers_per_account
+                body = 'Reached container limit of %s' % (
+                    self.app.max_containers_per_account, )
+                resp = HTTPForbidden(request=req, body=body)
                 return resp
         container_partition, containers = self.app.container_ring.get_nodes(
             self.account_name, self.container_name)
@@ -356,6 +358,27 @@ class ContainerController(Controller):
             return HTTPNotFound(request=req)
         return resp
 
+    @private
+    def UPDATE(self, req):
+        """HTTP UPDATE request handler.
+
+        Method to perform bulk operations on container DBs,
+        similar to a merge_items REPLICATE request.
+
+        Not client facing; internal clients or middlewares must include
+        ``X-Backend-Allow-Method: UPDATE`` header to access.
+        """
+        container_partition, containers = self.app.container_ring.get_nodes(
+            self.account_name, self.container_name)
+        # Since this isn't client facing, expect callers to supply an index
+        policy_index = req.headers['X-Backend-Storage-Policy-Index']
+        headers = self._backend_requests(
+            req, len(containers), account_partition=None, accounts=[],
+            policy_index=policy_index)
+        return self.make_requests(
+            req, self.app.container_ring, container_partition, 'UPDATE',
+            req.swift_entity_path, headers, body=req.body)
+
     def _backend_requests(self, req, n_outgoing, account_partition, accounts,
                           policy_index=None):
         additional = {'X-Timestamp': Timestamp.now().internal}
