@@ -32,6 +32,7 @@ from swiftclient import get_auth
 
 from swift.common import constraints
 from swift.common.http import is_success
+from swift.common.swob import str_to_wsgi, wsgi_to_str
 from swift.common.utils import config_true_value
 
 from test import safe_repr
@@ -198,18 +199,16 @@ def putrequest(self, method, url, skip_host=False, skip_accept_encoding=False):
 
 class Connection(object):
     def __init__(self, config):
-        for key in 'auth_host auth_port auth_ssl username password'.split():
+        for key in 'auth_uri username password'.split():
             if key not in config:
                 raise SkipTest(
                     "Missing required configuration parameter: %s" % key)
 
-        self.auth_host = config['auth_host']
-        self.auth_port = int(config['auth_port'])
-        self.auth_ssl = config['auth_ssl'] in ('on', 'true', 'yes', '1')
+        self.auth_url = config['auth_uri']
         self.insecure = config_true_value(config.get('insecure', 'false'))
-        self.auth_prefix = config.get('auth_prefix', '/')
         self.auth_version = str(config.get('auth_version', '1'))
 
+        self.domain = config.get('domain')
         self.account = config.get('account')
         self.username = config['username']
         self.password = config['password']
@@ -256,18 +255,10 @@ class Connection(object):
         return Account(self, self.account)
 
     def authenticate(self):
-        if self.auth_version == "1":
-            auth_path = '%sv1.0' % (self.auth_prefix)
-            if self.account:
-                auth_user = '%s:%s' % (self.account, self.username)
-            else:
-                auth_user = self.username
+        if self.auth_version == "1" and self.account:
+            auth_user = '%s:%s' % (self.account, self.username)
         else:
             auth_user = self.username
-            auth_path = self.auth_prefix
-        auth_scheme = 'https://' if self.auth_ssl else 'http://'
-        auth_netloc = "%s:%d" % (self.auth_host, self.auth_port)
-        auth_url = auth_scheme + auth_netloc + auth_path
 
         if self.insecure:
             try:
@@ -279,11 +270,16 @@ class Connection(object):
             else:
                 requests.packages.urllib3.disable_warnings(
                     InsecureRequestWarning)
+        if self.domain:
+            os_opts = {'project_domain_name': self.domain,
+                       'user_domain_name': self.domain}
+        else:
+            os_opts = {}
         authargs = dict(snet=False, tenant_name=self.account,
-                        auth_version=self.auth_version, os_options={},
+                        auth_version=self.auth_version, os_options=os_opts,
                         insecure=self.insecure)
         (storage_url, storage_token) = get_auth(
-            auth_url, auth_user, self.password, **authargs)
+            self.auth_url, auth_user, self.password, **authargs)
 
         if not (storage_url and storage_token):
             raise AuthenticationFailed()
@@ -335,7 +331,7 @@ class Connection(object):
         if path:
             quote = urllib.parse.quote
             if cfg.get('no_quote') or cfg.get('no_path_quote'):
-                quote = lambda x: x
+                quote = str_to_wsgi
             return '%s/%s' % (self.storage_path,
                               '/'.join([quote(i) for i in path]))
         else:
@@ -353,7 +349,8 @@ class Connection(object):
             headers['X-Auth-Token'] = cfg.get('use_token')
 
         if isinstance(hdrs, dict):
-            headers.update(hdrs)
+            headers.update((str_to_wsgi(h), str_to_wsgi(v))
+                           for h, v in hdrs.items())
         return headers
 
     def make_request(self, method, path=None, data=b'', hdrs=None, parms=None,
@@ -500,7 +497,10 @@ class Base(object):
                 'x-container-bytes-used',
             )
 
-        headers = dict(self.conn.response.getheaders())
+        # NB: on py2, headers are always lower; on py3, they match the bytes
+        # on the wire
+        headers = dict((wsgi_to_str(h).lower(), wsgi_to_str(v))
+                       for h, v in self.conn.response.getheaders())
         ret = {}
 
         for return_key, header in required_fields:
@@ -965,17 +965,19 @@ class File(Base):
             raise ResponseError(self.conn.response, 'HEAD',
                                 self.conn.make_path(self.path))
 
-        for hdr in self.conn.response.getheaders():
-            if hdr[0].lower() == 'content-type':
-                self.content_type = hdr[1]
-            if hdr[0].lower().startswith('x-object-meta-'):
-                self.metadata[hdr[0][14:]] = hdr[1]
-            if hdr[0].lower() == 'etag':
-                self.etag = hdr[1]
-            if hdr[0].lower() == 'content-length':
-                self.size = int(hdr[1])
-            if hdr[0].lower() == 'last-modified':
-                self.last_modified = hdr[1]
+        for hdr, val in self.conn.response.getheaders():
+            hdr = wsgi_to_str(hdr).lower()
+            val = wsgi_to_str(val)
+            if hdr == 'content-type':
+                self.content_type = val
+            if hdr.startswith('x-object-meta-'):
+                self.metadata[hdr[14:]] = val
+            if hdr == 'etag':
+                self.etag = val
+            if hdr == 'content-length':
+                self.size = int(val)
+            if hdr == 'last-modified':
+                self.last_modified = val
 
         return True
 
@@ -1018,11 +1020,11 @@ class File(Base):
             raise ResponseError(self.conn.response, 'GET',
                                 self.conn.make_path(self.path))
 
-        for hdr in self.conn.response.getheaders():
-            if hdr[0].lower() == 'content-type':
-                self.content_type = hdr[1]
-            if hdr[0].lower() == 'content-range':
-                self.content_range = hdr[1]
+        for hdr, val in self.conn.response.getheaders():
+            if hdr.lower() == 'content-type':
+                self.content_type = wsgi_to_str(val)
+            if hdr.lower() == 'content-range':
+                self.content_range = val
 
         if hasattr(buffer, 'write'):
             scratch = self.conn.response.read(8192)
