@@ -129,7 +129,8 @@ class MockBarbicanKeyManager(object):
             raise ValueError(ERR_MESSAGE_SECRET_INCORRECTLY_SPECIFIED)
         elif key_id == TEST_KMS_NONE_KEY_ID:
             return None
-        return MockBarbicanKey(b'x' * 32, key_id)
+        key_str = (str(key_id[0]) * 32).encode('utf8')
+        return MockBarbicanKey(key_str, key_id)
 
 
 class MockBarbicanKey(object):
@@ -189,7 +190,7 @@ class TestKmsKeymaster(unittest.TestCase):
     Tests using the v3 Identity API, where all calls to Barbican are mocked.
     """
 
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
     @mock.patch.object(kms_keymaster.KmsKeyMaster,
                        '_get_root_secret')
     def test_filter_v3(self, mock_get_root_secret_from_kms,
@@ -201,7 +202,7 @@ class TestKmsKeymaster(unittest.TestCase):
         self.assertTrue(callable(factory))
         self.assertTrue(callable(factory(self.swift)))
 
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
     @mock.patch.object(kms_keymaster.KmsKeyMaster,
                        '_get_root_secret')
     def test_app_exception_v3(self, mock_get_root_secret_from_kms,
@@ -215,35 +216,47 @@ class TestKmsKeymaster(unittest.TestCase):
         start_response, _ = capture_start_response()
         self.assertRaises(Exception, app, req.environ, start_response)
 
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
     @mock.patch.object(kms_keymaster.KmsKeyMaster, '_get_root_secret')
     def test_get_root_secret(
-            self, mock_get_root_secret_from_kms, mock_readconf):
+            self, mock_get_root_secret_from_kms):
         # Successful call with coarse _get_root_secret_from_kms() mock.
         mock_get_root_secret_from_kms.return_value = (
             base64.b64encode(b'x' * 32))
-        '''
-        Return valid Barbican configuration parameters.
-        '''
-        mock_readconf.return_value = TEST_KMS_KEYMASTER_CONF
-        '''
-        Verify that keys are derived correctly by the keymaster.
-        '''
+        # Provide valid Barbican configuration parameters in proxy-server
+        # config.
         self.app = kms_keymaster.KmsKeyMaster(self.swift,
                                               TEST_KMS_KEYMASTER_CONF)
-        '''
-        Verify that _get_root_secret_from_kms() was called with the
-        correct parameters.
-        '''
+        # Verify that _get_root_secret_from_kms() was called with the
+        # correct parameters.
         mock_get_root_secret_from_kms.assert_called_with(
             TEST_KMS_KEYMASTER_CONF
         )
 
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
+    @mock.patch.object(kms_keymaster.KmsKeyMaster, '_get_root_secret')
+    def test_get_root_secret_from_external_file(
+            self, mock_get_root_secret_from_kms, mock_readconf):
+        # Return valid Barbican configuration parameters.
+        mock_readconf.return_value = TEST_KMS_KEYMASTER_CONF
+        # Successful call with coarse _get_root_secret_from_kms() mock.
+        mock_get_root_secret_from_kms.return_value = (
+            base64.b64encode(b'x' * 32))
+        # Point to external config in proxy-server config.
+        self.app = kms_keymaster.KmsKeyMaster(
+            self.swift, TEST_PROXYSERVER_CONF_EXTERNAL_KEYMASTER_CONF)
+        # Verify that _get_root_secret_from_kms() was called with the
+        # correct parameters.
+        mock_get_root_secret_from_kms.assert_called_with(
+            TEST_KMS_KEYMASTER_CONF
+        )
+        self.assertEqual(mock_readconf.mock_calls, [
+            mock.call('PATH_TO_KEYMASTER_CONFIG_FILE', 'kms_keymaster')])
+
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.'
                 'keystone_password.KeystonePassword')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
     def test_mocked_castellan_keymanager(
             self, mock_castellan_key_manager, mock_readconf,
@@ -279,7 +292,7 @@ class TestKmsKeymaster(unittest.TestCase):
                 'keystone_password.KeystonePassword')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
     def test_mocked_castellan_keymanager_invalid_key_id(
             self, mock_castellan_key_manager, mock_readconf,
@@ -312,7 +325,7 @@ class TestKmsKeymaster(unittest.TestCase):
                 self.swift, TEST_PROXYSERVER_CONF_EXTERNAL_KEYMASTER_CONF)
             raise Exception('Success even though key id invalid')
         except ValueError as e:
-            self.assertEqual(e.message,
+            self.assertEqual(e.args[0],
                              ERR_MESSAGE_SECRET_INCORRECTLY_SPECIFIED)
         except Exception:
             print("Unexpected error: %s" % sys.exc_info()[0])
@@ -322,7 +335,7 @@ class TestKmsKeymaster(unittest.TestCase):
                 'keystone_password.KeystonePassword')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
     def test_mocked_castellan_keymanager_nonexistent_key_id(
             self, mock_castellan_key_manager, mock_readconf,
@@ -357,13 +370,13 @@ class TestKmsKeymaster(unittest.TestCase):
         except Exception as e:
             expected_message = ('Key not found, uuid: ' +
                                 TEST_KMS_NONEXISTENT_KEY_ID)
-            self.assertEqual(e.message, expected_message)
+            self.assertEqual(e.args[0], expected_message)
 
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.'
                 'keystone_password.KeystonePassword')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
     def test_mocked_castellan_keymanager_invalid_key_format(
             self, mock_castellan_key_manager, mock_readconf,
@@ -405,7 +418,7 @@ class TestKmsKeymaster(unittest.TestCase):
                 'keystone_password.KeystonePassword')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
     def test_mocked_castellan_keymanager_config_file_and_params(
             self, mock_castellan_key_manager, mock_readconf,
@@ -441,7 +454,7 @@ class TestKmsKeymaster(unittest.TestCase):
         except Exception as e:
             expected_message = ('keymaster_config_path is set, but there are '
                                 'other config options specified:')
-            self.assertTrue(e.message.startswith(expected_message),
+            self.assertTrue(e.args[0].startswith(expected_message),
                             "Error message does not start with '%s'" %
                             expected_message)
 
@@ -449,7 +462,7 @@ class TestKmsKeymaster(unittest.TestCase):
                 'keystone_password.KeystonePassword')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
     def test_mocked_castellan_keymanager_invalid_username(
             self, mock_castellan_key_manager, mock_readconf,
@@ -491,7 +504,7 @@ class TestKmsKeymaster(unittest.TestCase):
                 'keystone_password.KeystonePassword')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
     def test_mocked_castellan_keymanager_invalid_password(
             self, mock_castellan_key_manager, mock_readconf,
@@ -533,7 +546,7 @@ class TestKmsKeymaster(unittest.TestCase):
                 'keystone_password.KeystonePassword')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
     def test_mocked_castellan_keymanager_connect_failure_auth_url(
             self, mock_castellan_key_manager, mock_readconf,
@@ -574,7 +587,7 @@ class TestKmsKeymaster(unittest.TestCase):
                 'keystone_password.KeystonePassword')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
     def test_mocked_castellan_keymanager_bad_auth_url(
             self, mock_castellan_key_manager, mock_readconf,
@@ -616,7 +629,7 @@ class TestKmsKeymaster(unittest.TestCase):
                 'keystone_password.KeystonePassword')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
     def test_mocked_castellan_keymanager_bad_user_domain_name(
             self, mock_castellan_key_manager, mock_readconf,
@@ -658,7 +671,7 @@ class TestKmsKeymaster(unittest.TestCase):
                 'keystone_password.KeystonePassword')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
     def test_mocked_castellan_keymanager_invalid_key_algorithm(
             self, mock_castellan_key_manager, mock_readconf,
@@ -700,7 +713,7 @@ class TestKmsKeymaster(unittest.TestCase):
                 'keystone_password.KeystonePassword')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
     def test_mocked_castellan_keymanager_invalid_key_length(
             self, mock_castellan_key_manager, mock_readconf,
@@ -742,7 +755,7 @@ class TestKmsKeymaster(unittest.TestCase):
                 'keystone_password.KeystonePassword')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
-    @mock.patch('swift.common.middleware.crypto.kms_keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
     def test_mocked_castellan_keymanager_none_key(
             self, mock_castellan_key_manager, mock_readconf,
@@ -780,6 +793,70 @@ class TestKmsKeymaster(unittest.TestCase):
             print("Unexpected error: %s" % sys.exc_info()[0])
             raise
 
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.'
+                'keystone_password.KeystonePassword', MockPassword)
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
+    def test_get_root_secret_multiple_keys(
+            self, mock_castellan_key_manager, mock_readconf,
+            mock_castellan_options, mock_oslo_config,):
+        config = dict(TEST_KMS_KEYMASTER_CONF)
+        config.update({
+            'key_id_foo': 'foo-valid_kms_key_id-123456',
+            'key_id_bar': 'bar-valid_kms_key_id-123456',
+            'active_root_secret_id': 'foo'})
+
+        # Set side_effect functions.
+        mock_castellan_key_manager.API.side_effect = (
+            mock_castellan_api_side_effect)
+        mock_castellan_options.set_defaults.side_effect = (
+            mock_options_set_defaults_side_effect)
+        mock_oslo_config.ConfigOpts.side_effect = (
+            mock_config_opts_side_effect)
+
+        # Return valid Barbican configuration parameters.
+        mock_readconf.return_value = config
+
+        self.app = kms_keymaster.KmsKeyMaster(self.swift,
+                                              config)
+
+        expected_secrets = {
+            None: b'vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv',
+            'foo': b'ffffffffffffffffffffffffffffffff',
+            'bar': b'bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb'}
+        self.assertDictEqual(self.app._root_secrets, expected_secrets)
+        self.assertEqual(self.app.active_secret_id, 'foo')
+
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.'
+                'keystone_password.KeystonePassword', MockPassword)
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
+    def test_get_root_secret_legacy_key_id(
+            self, mock_castellan_key_manager, mock_readconf,
+            mock_castellan_options, mock_oslo_config):
+
+        # Set side_effect functions.
+        mock_castellan_key_manager.API.side_effect = (
+            mock_castellan_api_side_effect)
+        mock_castellan_options.set_defaults.side_effect = (
+            mock_options_set_defaults_side_effect)
+        mock_oslo_config.ConfigOpts.side_effect = (
+            mock_config_opts_side_effect)
+
+        # Return valid Barbican configuration parameters.
+        mock_readconf.return_value = TEST_KMS_KEYMASTER_CONF
+
+        self.app = kms_keymaster.KmsKeyMaster(self.swift,
+                                              TEST_KMS_KEYMASTER_CONF)
+
+        expected_secrets = {None: b'vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv'}
+        self.assertDictEqual(self.app._root_secrets, expected_secrets)
+        self.assertIsNone(self.app.active_secret_id)
+
 
 if __name__ == '__main__':
     unittest.main()
