@@ -165,12 +165,16 @@ class FakeSwift(object):
                     # keep old sysmeta for s3acl
                     headers.update({key: value})
 
+        if body is not None and not isinstance(body, (bytes, list)):
+            body = body.encode('utf8')
         self._responses[(method, path)] = (response_class, headers, body)
 
     def register_unconditionally(self, method, path, response_class, headers,
                                  body):
         # register() keeps old sysmeta around, but
         # register_unconditionally() keeps nothing.
+        if body is not None and not isinstance(body, bytes):
+            body = body.encode('utf8')
         self._responses[(method, path)] = (response_class, headers, body)
 
     def clear_calls(self):
