@@ -38,6 +38,7 @@ from swift.obj.diskfile import (
 from swift.common.ring import RingData
 from swift.common import utils
 from swift.common.header_key_dict import HeaderKeyDict
+from swift.common.swob import bytes_to_wsgi
 from swift.common.utils import (
     hash_path, normalize_timestamp, mkdirs, write_pickle)
 from swift.common.storage_policy import StoragePolicy, POLICIES
@@ -489,7 +490,7 @@ class TestObjectUpdater(unittest.TestCase):
         self.assertTrue(os.path.exists(op_path))
         self.assertEqual(ou.logger.get_increment_counts(),
                          {'failures': 1, 'unlinks': 1})
-        self.assertIsNone(pickle.load(open(op_path)).get('successes'))
+        self.assertIsNone(pickle.load(open(op_path, 'rb')).get('successes'))
 
         bindsock = listen_zero()
 
@@ -498,20 +499,20 @@ class TestObjectUpdater(unittest.TestCase):
                 with Timeout(3):
                     inc = sock.makefile('rb')
                     out = sock.makefile('wb')
-                    out.write('HTTP/1.1 %d OK\r\nContent-Length: 0\r\n\r\n' %
+                    out.write(b'HTTP/1.1 %d OK\r\nContent-Length: 0\r\n\r\n' %
                               return_code)
                     out.flush()
                     self.assertEqual(inc.readline(),
-                                     'PUT /sda1/0/a/c/o HTTP/1.1\r\n')
+                                     b'PUT /sda1/0/a/c/o HTTP/1.1\r\n')
                     headers = HeaderKeyDict()
-                    line = inc.readline()
+                    line = bytes_to_wsgi(inc.readline())
                     while line and line != '\r\n':
                         headers[line.split(':')[0]] = \
                             line.split(':')[1].strip()
-                        line = inc.readline()
-                    self.assertTrue('x-container-timestamp' in headers)
-                    self.assertTrue('X-Backend-Storage-Policy-Index' in
-                                    headers)
+                        line = bytes_to_wsgi(inc.readline())
+                    self.assertIn('x-container-timestamp', headers)
+                    self.assertIn('X-Backend-Storage-Policy-Index',
+                                  headers)
             except BaseException as err:
                 return err
             return None
@@ -546,7 +547,7 @@ class TestObjectUpdater(unittest.TestCase):
         self.assertEqual(ou.logger.get_increment_counts(),
                          {'failures': 1})
         self.assertEqual([0],
-                         pickle.load(open(op_path)).get('successes'))
+                         pickle.load(open(op_path, 'rb')).get('successes'))
 
         event = spawn(accept, [404, 201])
         ou.logger._clear()
@@ -558,7 +559,7 @@ class TestObjectUpdater(unittest.TestCase):
         self.assertEqual(ou.logger.get_increment_counts(),
                          {'failures': 1})
         self.assertEqual([0, 2],
-                         pickle.load(open(op_path)).get('successes'))
+                         pickle.load(open(op_path, 'rb')).get('successes'))
 
         event = spawn(accept, [201])
         ou.logger._clear()
@@ -867,7 +868,7 @@ class TestObjectUpdater(unittest.TestCase):
         self.assertEqual([mock.ANY], async_files)
         async_path = os.path.join(
             async_dir, async_subdirs[0], async_files[0])
-        with open(async_path) as fd:
+        with open(async_path, 'rb') as fd:
             async_data = pickle.load(fd)
         return async_path, async_data
 
